/*
     "ish.h" ish for UNIX
*/
#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <ctype.h>
#include <errno.h>
#include <string.h>
#include <fcntl.h>

#ifdef __WIN32__
#define NOINDEX
#define TZSET
#define NOBCOPY
#define MBFNAME
#define LONGNAME
#define NAMELOWER
#include <windows.h>
#endif

#ifdef __LINUX__
#define NOINDEX
#define TZSET
#define NOBCOPY
#define LONGNAME
#define NAMELOWER
#endif

#ifndef NOINDEX
#include <strings.h>
#endif

#ifdef TZSET
#include <time.h>
#else
#include <sys/time.h>
#endif

#ifdef FTIME
#include <sys/timeb.h>
#endif

#ifdef NDIR
#include <sys/ndir.h>
#define    dirent    direct
#else
#include <dirent.h>
#endif

#define VERSION "Ver2.01b1"

#define JIS7_L	63
#define JIS8_L	69
#define SJIS_L	73
#define OJIS_L	69
#define	NJIS_L	69

#ifdef __MSDOS__
#define PATHSP	'\\'
#else
#define PATHSP	'/'
#endif

#ifdef LONGNAME
#define NAMLEN	 260
#define LBUFLEN	 320
#else
#define NAMLEN	 40
#define LBUFLEN	 81
#endif

#ifndef O_BINARY
#define O_BINARY 0
#endif

#define MS_DOS	0x00
#define CP_M	0x10
#define OS_9	0x20
#define OS_9EXT 0x21	/* Extended OS9 format */
#define UNIX	0x30	/* UNIX */
#define LINUX	0x31
#define MAC_OLD	0x40
#define	MAC		0x41
#define OTHER	0xFE
#define ALL_OS	0xFF

#define CREISH	0	/* Create ish file */
#define RESNEW	1
#define RESALL	2
#define RESASK	3

#define ON	0
#define OFF	(-1)

#define ERR	(-1)

#define MVIDLEN	96

#define chkcrc(argv,len)	(0x1d0f==calcrc(argv,len,0xffff))
#define setcrc(argv,len)	(setwrd(argv+len,~calcrc(argv,len,0xffff)))

#define unschr(chr) (chr)

#ifdef	NOINDEX
#ifdef MBFNAME
#include <mbstring.h>
#define index   _mbschr
#define rindex  _mbsrchr
#else
#define	index   strchr
#define	rindex  strrchr
#endif
#endif

#ifdef MBFNAME
#define CCODE	"CP932"
#else
#ifdef UTF8
#define CCODE	"UTF-8"
#else
#define CCODE	"EUC"
#endif
#endif

#ifdef NOBCOPY
#define bcopy(a,b,c)   memcpy(b,a,c)
#define bcmp(a,b,c)    memcmp((a),(b),(c))
#endif

enum EncDecMode { JIS7, JIS8, SJIS, OJIS, NJIS } ;
typedef unsigned char	uchar;
typedef unsigned short	ushort;
typedef unsigned int	uint;
typedef unsigned long	ulong;

typedef struct {
	unsigned char sinc;
	unsigned char itype;
	unsigned char fsize[4];
	unsigned char line[2];
	unsigned short block;
	unsigned char byte;
	unsigned char ibyte;
	unsigned char iname[11];
	unsigned char janle;
	unsigned char os;
	unsigned char exttype;
	unsigned char tstamp;
	unsigned char time[4];
	/* add by CEX */
	unsigned char dtime[3];	/* difference time */
	unsigned char fcrc16[2];	/* file CRC-16 */
	unsigned char fcrc32[4];	/* file CRC-32 */
	unsigned char bcnt[4];	/* byte count */
	unsigned char vcrc16[2];	/* volume CRC-16 */
	/* end CEX */
#if 0
	unsigned char atr;		/* 32:	OS-9Ext only */
	unsigned char ename[29];	/* 33-61: OS-9Ext only */
#endif
} ish_head;

typedef    struct {
	unsigned char    fsize[4];
	unsigned char    line[2];
	unsigned char    block[2];
	unsigned char    byte;
	unsigned char    ibyte;
	unsigned char    fname[11];
	unsigned char    janle;
	unsigned char    os;
	unsigned char    exttype;
	unsigned char    tstamp;
	unsigned char    time[4];
	unsigned char    dtime[3];
	unsigned char    fcrc16[2];
	unsigned char    fcrc32[4];
	unsigned char    bcnt[4];
	unsigned char    reserved[54];
} HEADER_COPY;

struct IDBLOCK {
	unsigned char    id_block_size[2];
	unsigned char    id_block_version_number[2];
	unsigned char    id_block_id_offset[2];
	unsigned char    id_block_id_size[2];
	unsigned char    header_copy_offset[2];
	unsigned char    header_copy_size[2];
	unsigned char    sequence_number_table_offset[2];
	unsigned char    sequence_number_table_size[2];
	unsigned char    reserved[16];
	unsigned char    id_block_id[MVIDLEN];
	HEADER_COPY      header_copy;
	unsigned char    sequence_number_table[32];
	unsigned char    id_block_offset[4];
};

typedef struct TMPCHAIN {
	uchar	target[NAMLEN+16];
	uchar	tmpo[NAMLEN+16];
	struct TMPCHAIN *next;
} tmpchain;

/* extern valiable */
extern FILE *opath,*ipath;
extern char name[NAMLEN+16];

extern short int jis,os,esc,title,mode;
extern int path,size;
extern long lsize;
extern char *filename;

#ifdef __WIN32__
extern uchar sname[NAMLEN];
extern uchar *sfilename;
#endif

extern uchar buff[LBUFLEN];
extern uchar obuf[LBUFLEN];
extern uchar tatesum[LBUFLEN];
extern uchar yokosum[LBUFLEN];
extern uchar atrb;
extern uchar myID[MVIDLEN];

extern enum EncDecMode edmode;

// ish.c
extern void __exit(int rc);
extern void help(void);
extern void puttitle(void);

// decode.c
#ifdef LONGNAME
extern char *strrstr(const char *str, const char *key);
#endif
extern int ismbcwc(const unsigned char *str, int n);
extern int decode(int oflg);
extern void hedprn(ish_head *head);
extern int decish(void);
extern int decblk(void);
extern int dec_j(uchar *argv, int len);
extern int ecc(int err, int e1, int e2);
extern int readln(FILE *ipath, char *buff, int maxlen);
extern int readln2(FILE *ipath, char *buff,int maxlen);
extern int check_overwrite(char *outfile);
extern int days(struct tm *t);
extern time_t seconds(struct tm *t);
extern int topchar(uchar c);

// encode.c
extern int encode(int oflg);
extern int encode1(int oflg);
extern void encd(char *argv);
extern void hdprint(void);
extern void sethed(void);
extern void setwrd(uchar *adr,int dat);
extern void print(char *argv);
extern void prtitle(void);
extern void chksum(int no, int len, uchar *argv);

// mvol.c
extern tmpchain *root;
extern void init_mhead(char *head);
extern int mvol_name(char *mvname);
extern int mvol_ok(char *mvname,int n);
extern tmpchain *search_tmpo(char *mvname);
extern void restore_done(char *mvname,long sz);

// mencode.c
extern int mencode(int oflg);
long calcline(int max,long len,long sw);

// crc.c
ushort calcrc(uchar *arg, int len, ushort crc);
// crc32.c
ulong calcrc32(uchar *arg, int len, ulong crc);
// jis7.c
extern void enc_jis7(uchar *argv, int len);
extern int dec_jis7(uchar *argv, int len);
// jis8.c
extern void enc_jis8(uchar *argv, int len);
extern int dec_jis8(uchar *argv, int len);
// njis.,c
extern void enc_njis(uchar *argv, int len);
extern int ins14(unsigned d);
extern int dec_njis(uchar *argv, int len);
// sjis.c
extern void enc_sjis(uchar *argv, int len);
extern int dec_sjis(uchar *argv, int len);
